#include <gnome.h>

#include <gst/gst.h>
#include <gst/gstutils.h>

#include "gsteditor.h"
#include "gsteditorelement.h"

static void gst_editor_element_class_init(GstEditorElementClass *klass);
static void gst_editor_element_init(GstEditorElement *element);
static void gst_editor_element_set_arg(GtkObject *object,GtkArg *arg,guint id);
static void gst_editor_element_get_arg(GtkObject *object,GtkArg *arg,guint id);

static void gst_editor_element_update(GnomeCanvasItem *item,double *affine,
                                      ArtSVP *clip_path,int flags);
static gint gst_editor_element_event(GtkWidget *widget,GdkEvent *event);

gint gst_editor_element_border_event(GnomeCanvasItem *item,GdkEvent *event,
                                     GstEditorElement *element);


enum {
  ARG_0,
  ARG_XPOS,
  ARG_YPOS,
  ARG_WIDTH,  
  ARG_HEIGHT,
  ARG_TITLE,
  ARG_FACTORY,
};

static GnomeCanvasGroupClass *parent_class;

GtkType gst_editor_element_get_type() {
  static GtkType element_type = 0;

  if (!element_type) {
    static const GtkTypeInfo element_info = {
      "GstEditorElement",
      sizeof(GstEditorElement),
      sizeof(GstEditorElementClass),
      (GtkClassInitFunc)gst_editor_element_class_init,
      (GtkObjectInitFunc)gst_editor_element_init,
      NULL,
      NULL,
      (GtkClassInitFunc)NULL,
    };
    element_type = gtk_type_unique(gnome_canvas_group_get_type(),&element_info);
  }
  return element_type;
}

static void gst_editor_element_class_init(GstEditorElementClass *klass) {
  GnomeCanvasItemClass *item_class;
  GtkWidgetClass *widget_class;
  GtkObjectClass *object_class;

  item_class = (GnomeCanvasItemClass*)klass;
  widget_class = (GtkWidgetClass*)klass;
  object_class = (GtkObjectClass*)klass;

  parent_class = gtk_type_class(gnome_canvas_group_get_type());

  gtk_object_add_arg_type("GstEditorElement::xpos",GTK_TYPE_DOUBLE,
                          GTK_ARG_READWRITE,ARG_XPOS);
  gtk_object_add_arg_type("GstEditorElement::ypos",GTK_TYPE_DOUBLE,
                          GTK_ARG_READWRITE,ARG_YPOS);
  gtk_object_add_arg_type("GstEditorElement::width",GTK_TYPE_DOUBLE,
                          GTK_ARG_READWRITE,ARG_WIDTH);
  gtk_object_add_arg_type("GstEditorElement::height",GTK_TYPE_DOUBLE,
                          GTK_ARG_READWRITE,ARG_HEIGHT);
  gtk_object_add_arg_type("GstEditorElement::factory",GTK_TYPE_POINTER,
                          GTK_ARG_READWRITE|GTK_ARG_CONSTRUCT_ONLY,
                          ARG_FACTORY);

  klass->group_set_arg = object_class->set_arg;
  klass->group_get_arg = object_class->get_arg;

  item_class->update = gst_editor_element_update;

  widget_class->event = gst_editor_element_event;

  object_class->set_arg = gst_editor_element_set_arg;
  object_class->get_arg = gst_editor_element_get_arg;
}

static void gst_editor_element_init(GstEditorElement *element) {
  element->factory = NULL;

  element->created = FALSE;
  element->x = 0.0;element->y = 0.0;
  element->width = 0.0;element->height = 00;
  element->name = g_strdup("temp");
  element->resized = FALSE;
}

static void gst_editor_element_set_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstEditorElement *element;
  GnomeCanvasGroup *group;

  /* get the major types of this object */
  element = GST_EDITOR_ELEMENT(object);
  group = GNOME_CANVAS_GROUP(object);

  switch (id) {
    /* these currently do nothing but resize the border */
    /* they will eventually do resizing of the whole editor_element */
    case ARG_XPOS:
      gtk_object_set(GTK_OBJECT(group),"x",GTK_VALUE_DOUBLE(*arg),NULL);
      break;
    case ARG_YPOS:
      gtk_object_set(GTK_OBJECT(group),"y",GTK_VALUE_DOUBLE(*arg),NULL);
      break;
    case ARG_WIDTH:
      element->width = GTK_VALUE_DOUBLE(*arg);
      element->resized = TRUE;
      break;
    case ARG_HEIGHT:
      element->height = GTK_VALUE_DOUBLE(*arg);
      element->resized = TRUE;
      break;
    case ARG_FACTORY:
      element->factory = GTK_VALUE_POINTER(*arg);
      element->name = element->factory->name;
      break;
    default:
      g_warning("gsteditorelement: unknown arg!");
      break;
  }
  gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(object));
}

static void gst_editor_element_get_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstEditorElement *element;
  GnomeCanvasGroup *group;
  GtkObjectClass *border_objectclass;

  /* get the major types of this object */
  element = GST_EDITOR_ELEMENT(object);
  group = GNOME_CANVAS_GROUP(object);

  /* extract the GtkObjectClass of the border, so we can chain to it */
  border_objectclass = (GtkObjectClass *)(GTK_OBJECT(element->border)->klass);

  switch (id) {
    default:
      arg->type = GTK_TYPE_INVALID;
      break;
  }
}

static void gst_editor_element_update(GnomeCanvasItem *item,double *affine,
                                      ArtSVP *clip_path,int flags) {
  GstEditorElement *element = GST_EDITOR_ELEMENT(item);

  /* first see if we have to go create everything first */
  if (element->created == FALSE) {
    GnomeCanvasGroup *selfgroup = GNOME_CANVAS_GROUP(element);
    element->border = gnome_canvas_item_new(selfgroup,
      gnome_canvas_rect_get_type(),
      "width_pixels",2,"fill_color","white","outline_color","black", 
      "x1",-(element->width/2),
      "y1",-(element->height/2),
      "x2", (element->width/2),
      "y2", (element->height/2),
      NULL);
    gtk_signal_connect(GTK_OBJECT(element->border),"event",
            	GTK_SIGNAL_FUNC(gst_editor_element_border_event),element);
    element->title = gnome_canvas_item_new(selfgroup,
      gnome_canvas_text_get_type(),
      "text",element->name,
      "x",-(element->width/2) + 2,
      "y",-(element->height/2) + 2,
      "anchor",GTK_ANCHOR_NORTH_WEST,
      NULL);
    element->resized = TRUE;
  }

  /* if we've been bumped for some reason */
  if (element->resized) {
    gdouble textwidth;
    textwidth = gst_util_get_double_arg(GTK_OBJECT(element->title),"text_width");
    g_print("text width is %f\n",textwidth);
    if ((textwidth + 4) < element->width)
      element->width = textwidth + 4;
    g_print("size is now %fx%f\n",element->width,element->height);
    /* start by resizing the bordering box */
    gtk_object_set(GTK_OBJECT(element->border),
                   "x1",-(element->width/2),
                   "y1",-(element->height/2),
                   "x2", (element->width/2),
                   "y2", (element->height/2),NULL);

    /* then move the text to the new top left */
    gtk_object_set(GTK_OBJECT(element->title),
                   "x",-(element->width/2) + 2,
                   "y",-(element->height/2) + 2,NULL);
  }

  if (GNOME_CANVAS_ITEM_CLASS(parent_class)->update)
    (* GNOME_CANVAS_ITEM_CLASS(parent_class)->update) (item, affine, clip_path, flags);
}

gint gst_editor_element_event(GtkWidget *widget,GdkEvent *event) {
  GstEditorElement *element = GST_EDITOR_ELEMENT(widget);

  g_print("got event on element %p\n",element);

  switch(event->type) {
    case GDK_ENTER_NOTIFY:
      g_print("entered element\n");
      break;
    case GDK_LEAVE_NOTIFY:
      g_print("left element\n");
      break;
    default:
      break;
  }
  return TRUE;
}

gint gst_editor_element_border_event(GnomeCanvasItem *item,GdkEvent *event,
                                     GstEditorElement *element) {
  switch(event->type) {
    case GDK_ENTER_NOTIFY:
      g_print("entered element border\n");
      g_print("in_child is now %d\n",
      ++GST_EDITOR(item->canvas)->in_child);
      break;
    case GDK_LEAVE_NOTIFY:
      g_print("left element border\n");
      g_print("in_child is now %d\n",
      --GST_EDITOR(item->canvas)->in_child);
      break;
    case GDK_MOTION_NOTIFY:
      g_print("moved inside the element\n");
      break;
    case GDK_BUTTON_PRESS:
      g_print("pressed button\n");
      break;
    default:
      break;
  }
  return TRUE;
}
