#include <gtk/gtk.h>
#include <gnome.h>
#include <libgnomeui/gnome-canvas.h>
#include <glade/glade.h>

#include <gst/gst.h>

#define GST_EDITOR(x) ((GstEditor *)(x))
#define GST_EDITOR_ELEMENT(x) ((GstEditorElement *)(x))

typedef struct _GstEditor GstEditor;
struct _GstEditor {
  GstBin *bin;

  GtkWidget *canvas;
  GnomeCanvasGroup *link_group;
  GnomeCanvasGroup *element_group;

  gboolean create_element;
  gboolean global_focus;

  GList *elements;
};
typedef struct _GstEditorElement GstEditorElement;
struct _GstEditorElement {
  /* parent editor */
  GstEditor *editor;
  /* GstElement this thing is a front end for */
  GstElement *element;

  gboolean creating;
  gboolean predrag;
  gboolean dragging;

  GnomeCanvasItem *item;
  GtkWidget *frame;
};

gint destroy(GtkWidget *widget,GdkEvent *event,gpointer data) {
  gtk_main_quit();
}

gint element_event(GnomeCanvasItem *item,GdkEvent *event,
                   GstEditorElement *element) {
  g_print("element %p got event %d: ",element,event->type);
  switch(event->type) {
    case GDK_ENTER_NOTIFY:
      g_print("entered object, grabbing LEAVE, PRESS, RELEASE, MOTION\n");
      gnome_canvas_item_grab(item,
                             GDK_LEAVE_NOTIFY_MASK |
                             GDK_BUTTON_PRESS_MASK |
                             GDK_BUTTON_RELEASE_MASK |
                             GDK_POINTER_MOTION_MASK,
                             NULL,event->button.time);
      element->predrag = TRUE;
      element->editor->create_element = FALSE;
      break;
    case GDK_LEAVE_NOTIFY:
      g_print("left object\n");
      if (element->predrag) {
        gnome_canvas_item_ungrab(item,event->button.time);
        element->predrag = FALSE;
        element->editor->create_element = TRUE;
      }
      break;
    case GDK_BUTTON_PRESS:
      g_print("pressed button %d\n",event->button.button);
      if (event->button.button == 1) {
        if (element->creating) {
          g_print("still creating the thing\n");
          element->creating = FALSE;
        } else {
          g_print("grabbing focus\n");
          gnome_canvas_item_grab(item,
                                 GDK_POINTER_MOTION_MASK |
                                 GDK_BUTTON_RELEASE_MASK,
                                 NULL,event->button.time);
          element->dragging = TRUE;
        }
      }
      break;
    case GDK_BUTTON_RELEASE:
      g_print("released button %d\n",event->button.button);
      if (event->button.button == 1) {
        if (element->dragging) {
          gnome_canvas_item_ungrab(item,event->button.time);
          element->dragging = FALSE;
        }
      }
      break;
    case GDK_MOTION_NOTIFY:
      g_print("moved cursor\n");
      if (element->dragging) {
        double newx, newy, dx, dy;
        newx = event->motion.x;
        newy = event->motion.y;
        dx = 0.0;dy = 0.0;

        //gnome_canvas_item_w2i(element,&newx,&newy);
        g_print("moving to %f,%f\n",newx,newy);
      }
      break;
    default:
      break;
  }

  return FALSE;
}

GstEditorElement *gst_editor_element_new(GstEditor *editor,
                                         gchar *name,
                                         gdouble x,gdouble y) {
  GstEditorElement *element;
  GtkWidget *button;

  element = GST_EDITOR_ELEMENT(malloc(sizeof(GstEditorElement)));

  /* we start with sane defaults */
  element->editor = editor;
  element->element = NULL;
  element->creating = TRUE;
  element->predrag = FALSE;
  element->dragging = FALSE;

  /* create the canvas objects */
  button = gtk_button_new_with_label(name);
  element->frame = gtk_frame_new(name);
  g_print("element->frame is %p\n",element->frame);
  gtk_container_add(GTK_CONTAINER(element->frame),GTK_WIDGET(button));

  element->item = gnome_canvas_item_new(editor->element_group,
                            gnome_canvas_rect_get_type(),
                            "x1",x,"y1",y,
                            "x2",x+100.0,"y2",y+40.0,
                            "width_pixels",5,
                            NULL);
  gtk_widget_show(button);
  gtk_widget_show(element->frame);
  g_print("element->item is %p\n",element->item);
  gtk_object_set_data(GTK_OBJECT(element->item),"element",
                      element);
  gtk_signal_connect(GTK_OBJECT(element->item),"event",
                     GTK_SIGNAL_FUNC(element_event),element);

/*  element->item = gnome_canvas_item_new(editor->element_group,
    gnome_canvas_text_get_type(),"x",x,"y",y,"text","GstElement",NULL);*/

  return element;
}



gint editor_button_press(GtkWidget *widget,GdkEvent *event,
                         GstEditor *editor) {
  gdouble x,y;
  GstEditorElement *element;

  g_print("got button press in editor\n");
}
/*
  gnome_canvas_window_to_world(GNOME_CANVAS(editor->canvas),
                               event->button.x,event->button.y,
                               &x,&y);
  element = gst_editor_element_new(editor,"GstElement",x,y);
  editor->elements = g_list_prepend(editor->elements,element);
  return TRUE;
}
*/

GtkWidget *create_canvas() {
  return GTK_WIDGET(gnome_canvas_new());
}

gint item_event(GnomeCanvasItem *item,GdkEvent *event,
                GstEditor *editor) {
  int retval;
  g_print("item %p got event %d\n",item,event->type);
  if (event->type == GDK_ENTER_NOTIFY) {
//    retval = gnome_canvas_item_grab(item,
//GDK_LEAVE_NOTIFY_MASK|GDK_POINTER_MOTION_MASK,
//                           NULL,GDK_CURRENT_TIME);
//    g_print("mouse entered %p, grabbing, got %d\n",item,retval);
    editor->global_focus = FALSE;
  } else if (event->type == GDK_LEAVE_NOTIFY) {
//    g_print("mouse left %p, ungrabbing\n",item);
//    gnome_canvas_item_ungrab(item,GDK_CURRENT_TIME);
    editor->global_focus = TRUE;
  }
  return TRUE;
}

gint editor_event(GnomeCanvasItem *item,GdkEvent *event,
                   GstEditor *editor) {
  if (editor->global_focus != TRUE) return FALSE;
  g_print("editor %p got event %d\n",editor,event->type);
  if (event->type == GDK_BUTTON_PRESS) {
    double x,y;
    g_print("editor %p got button press at %d,%d\n",editor,
            event->button.x,event->button.y);
    gnome_canvas_window_to_world(GNOME_CANVAS(editor->canvas),
                                 event->button.x,event->button.y,
                                 &x,&y);
    item = gnome_canvas_item_new(gnome_canvas_root(editor->canvas),
                                 gnome_canvas_rect_get_type(),
                                 "x1", x-20.0,
                                 "y1", y-10.0,
                                 "x2", x+20.0,
                                 "y2", y+10.0,
                                 "fill_color", "white",
                                 "outline_color", "black",
                                 "width_units", 1.0,
                                 NULL);
    gtk_signal_connect (GTK_OBJECT(item),"event",
                        (GtkSignalFunc)item_event,editor);
  }
}


int main(int argc,char *argv[]) {
//  GladeXML *xml;
  GtkWidget *window;
  GstEditor *editor = GST_EDITOR(malloc(sizeof(GstEditor)));

  gtk_init(&argc,&argv);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(window),"destroy",
                     GTK_SIGNAL_FUNC(destroy),NULL);

  editor->create_element = TRUE;
  editor->canvas = gnome_canvas_new();
  gtk_widget_set_usize(editor->canvas,200,200);
  editor->element_group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(
                          gnome_canvas_root(GNOME_CANVAS(editor->canvas)),
                          gnome_canvas_group_get_type(),NULL));
  editor->global_focus = TRUE;


  gtk_signal_connect(GTK_OBJECT(editor->canvas),"event",
                     (GtkSignalFunc)editor_event,editor);

  gtk_container_add(GTK_CONTAINER(window), editor->canvas);

  gtk_widget_show(editor->canvas);
  gtk_widget_show(window);

  gtk_main();

  return(0);
}
