#include <gtk/gtk.h>
#include <gst/gst.h>
#include <glade/glade.h>
#include <math.h>

#define DEBUG(format,args...) g_print("DEBUG: " format,##args)

extern gboolean _gst_plugin_spew;

GladeXML *xml;
GstElement *bin, *tee, *src, *decoder, *sink;
GstElement *effect = NULL, *spectrum = NULL, *syna = NULL;
GtkWidget *filename_entry, *async_window, *slider, *effect_slider;
GtkWidget *effect_name;
GtkWidget *drawingarea;
GtkWidget *synaimage = 0;
GstPad *spectrumpad;
GtkAdjustment *adj, *effect_adj;
gulong length;
gdouble update;
guchar *effectname = NULL;

void async_window_destroy(GtkWidget *widget,gpointer data) {
  gtk_main_quit();
}

void load_button_clicked(GtkWidget *widget,gpointer data) {
  if (GST_FLAG_IS_SET(bin,GST_STATE_RUNNING))
    gst_element_stop(bin);
  gtk_object_set(GTK_OBJECT(src),"location",
                 gtk_entry_get_text(GTK_ENTRY(filename_entry)),NULL);
  length = 0;
//  length = gst_util_get_long_arg(GST_OBJECT(src),"length");
  gtk_adjustment_set_value(adj,0.0);
  gst_element_start(bin,0);
  gst_element_start(syna,0);
  synaimage = gst_util_get_widget_arg(syna,"widget");
  DEBUG("synaimage is %p\n",synaimage);
  gtk_box_pack_end(glade_xml_get_widget(xml,"spectrum_box"),synaimage,
                   FALSE,FALSE,0);
}

void reset_button_clicked(GtkWidget *widget,gpointer data) {
  gtk_entry_set_text(GTK_ENTRY(filename_entry),
                     gst_util_get_string_arg(GST_OBJECT(src),"location"));
}

gboolean async_idle_func(gpointer data) {
  gst_src_push(GST_SRC(src));
  return TRUE;
}

void play_button_clicked(GtkWidget *widget,gpointer *data) {
  g_idle_remove_by_data(src);
  g_idle_add(async_idle_func,src);
}

void stop_button_clicked(GtkWidget *widget,gpointer *data) {
  g_idle_remove_by_data(src);
}

void slider_moved(GtkAdjustment *adj,gpointer *data) {
  gulong offset;
  if (fabs(adj->value - update) < 0.00001) return;
  if (!length)
    length = gst_util_get_long_arg(GST_OBJECT(src),"length");
  offset = length * adj->value;
  gtk_object_set(GTK_OBJECT(src),"offset",offset,NULL);
}

void effect_toggled(GtkWidget *widget,gpointer *data) {
  int state = 0;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE)
    state = 1;
  if (effect)
    gtk_object_set(GTK_OBJECT(effect),"active",state,NULL);
}  

void effect_slider_moved(GtkAdjustment *adj,gpointer *data) {
  if (effect)
    gtk_object_set(GTK_OBJECT(effect),effectname,adj->value,NULL);
}

void menu_exit(GtkWidget *widget,gpointer *data) {
  gtk_main_quit();
}

void menu_about(GtkWidget *widget,gpointer *data) {
  g_print("yo, this be the GIST!\n");
}

void eof(GstSrc *src) {
  DEBUG("have EOF\n");
  g_idle_remove_by_data(src);
}

void handoff(GstElement *element,GstPad *pad) {
  gulong offset;
  gdouble val = 0.0;

  offset = gst_util_get_long_arg(GST_OBJECT(src),"offset");
  if (!length)
    length = gst_util_get_long_arg(GST_OBJECT(src),"length");
  if (length)
    val = (gdouble)offset / length;
  update = val;
  gtk_adjustment_set_value(adj,val);

/*
  if (spectrum) {
    gfloat volume;
    volume = (gfloat)gst_util_get_int_arg(GST_OBJECT(vumeter),"volume") / 32768.0;
//    g_print("volume is %d\n",volume);
    gtk_progress_bar_update(GTK_PROGRESS_BAR(vumeter_progress),volume);
  }
*/
}

void gist_spectrum_chain(GstPad *pad,GstBuffer *buf) {
  int i;
  guchar *data = buf->data;
  gdk_draw_rectangle(drawingarea->window,drawingarea->style->black_gc,
                     TRUE,0,0,75,25);
  for (i=0;i<buf->datasize;i++) {
    gdk_draw_rectangle(drawingarea->window,drawingarea->style->white_gc,
                       TRUE,i,32-data[i],1,data[i]);
//    g_print(" %d",data[i]);
  }
  gst_buffer_unref(buf);
}

int main(int argc,char *argv[]) {
  /* Glade stuff */

  /* GST stuff */
  guint16 type;
  GList *factories;
  GstElementFactory *srcfactory = NULL, *sinkfactory = NULL;
  GstElementFactory *decoderfactory = NULL, *effectfactory = NULL;
  GstElementFactory *spectrumfactory = NULL, *synafactory = NULL;
  guchar *filename;

  _gst_plugin_spew = TRUE;

  /* if we have a filename */
  if (argc >= 2)
    filename = argv[1];
  else
    filename = NULL;

  /* start up all the libraries we're using */
  gtk_init(&argc,&argv);
  gst_init(&argc,&argv);
  glade_init();

  g_print("ready to do some damage\n");

  /* load all the plugins */
  gst_plugin_load_all();
//  gst_plugin_load("libgstxa.so");
//  gst_plugin_load("libgstspectrum.so");
//  gst_plugin_load("libsmoothwave.so");
  
  g_print("loaded plugins\n");

  /* create the easy pipeline */
  bin = gst_bin_new("bin");
  DEBUG("created bin\n");

  tee = gst_tee_new("tee");
  DEBUG("created tee\n");

  srcfactory = gst_elementfactory_find("asyncdisksrc");
  DEBUG("src elementfactory is %p\n",srcfactory);
  src = gst_elementfactory_create(srcfactory,"src");
  DEBUG("created src\n");

  sinkfactory = gst_elementfactory_find("audiosink");
  DEBUG("sink elementfactory is %p\n",sinkfactory);
  sink = gst_elementfactory_create(sinkfactory,"sink");
  DEBUG("created sink\n");

  /* now it's time to get the decoder */
  type = gst_type_find_by_mime("audio/mp3");
  DEBUG("type is %d\n",type);
  factories = gst_type_get_sinks(type);
  if (factories != NULL)
    decoderfactory = (GstElementFactory *)factories->data;
  else {
    g_print("sorry, can't find anyone registered to sink 'mp3'\n");
    return 1;
  }
  decoder = gst_elementfactory_create(decoderfactory,"decoder");
  if (decoder == NULL) {
    g_print("sorry, couldn't create decoder\n");
    return 1;
  }
  DEBUG("created decoder\n");

  /* effect */
  if (argc >= 3)
    effectname = g_strdup("volume");
  if (effectname)
    effectfactory = gst_elementfactory_find(effectname);
  if (effectfactory)
    effect = gst_elementfactory_create(effectfactory,"effect");
  if (effect)
    DEBUG("created effect\n");

  spectrumfactory = gst_elementfactory_find("spectrum");
  if (spectrumfactory)
    spectrum = gst_elementfactory_create(spectrumfactory,"spectrum");
  if (spectrum) {
    g_print("spectrum name is '%s'\n",gst_object_get_name(GST_OBJECT(spectrum)));
    spectrumpad = gst_pad_new("sink",GST_PAD_SINK);
    gst_pad_set_chain_function(spectrumpad,gist_spectrum_chain);
    DEBUG("created spectrum\n");
  }

  synafactory = gst_elementfactory_find(argv[2]);
  if (synafactory)
    syna = gst_elementfactory_create(synafactory,"syna");
  if (syna) {
    DEBUG("created syna\n");
  }

  /* add objects to the main pipeline */
  gst_bin_add(GST_BIN(bin),src);
  gst_bin_add(GST_BIN(bin),decoder);
  if (effect)
    gst_bin_add(GST_BIN(bin),effect);
  gst_bin_add(GST_BIN(bin),sink);
  if (spectrum)
    gst_bin_add(GST_BIN(bin),spectrum);
  if (syna)
    gst_bin_add(GST_BIN(bin),syna);

  /* connect src to sink */
  gst_pad_connect(gst_element_get_pad(src,"src"),
                  gst_element_get_pad(decoder,"sink"));
  if (effect) {
    gst_pad_connect(gst_element_get_pad(decoder,"src"),
                    gst_element_get_pad(effect,"sink"));
    gst_pad_connect(gst_element_get_pad(effect,"src"),
                    gst_element_get_pad(tee,"sink"));
  } else {
    gst_pad_connect(gst_element_get_pad(decoder,"src"),
                    gst_element_get_pad(tee,"sink"));
  }

  gst_pad_connect(gst_element_get_pad(tee,gst_tee_new_pad(GST_TEE(tee))),
                  gst_element_get_pad(sink,"sink"));
  g_print("connected sink\n");

/*  if (spectrum) {
    gchar *name = gst_tee_new_pad(GST_TEE(tee));
    g_print("new pad is named '%s'\n",name);
    gst_pad_connect(gst_element_get_pad(tee,name),
                    gst_element_get_pad(spectrum,"sink"));
    gst_pad_connect(gst_element_get_pad(spectrum,"src"),spectrumpad);
    g_print("connected spectrum\n");
  }*/

  if (syna) {
    gchar *name = gst_tee_new_pad(GST_TEE(tee));
    g_print("new pad is named '%s'\n",name);
    gst_pad_connect(gst_element_get_pad(tee,name),
                    gst_element_get_pad(syna,"sink"));
    g_print("connected syna\n");
  }

  g_print("DONE CREATING PIPELINE\n");

  /* create the GUI and find some of the useful widgets */
  xml = glade_xml_new("gist.glade",NULL);
g_print("loaded xml\n");
  filename_entry = glade_xml_get_widget(xml,"filename_entry");
  async_window = glade_xml_get_widget(xml,"async_window");
g_print("have xml, filename, async\n");
  slider = glade_xml_get_widget(xml,"slider");
  {
    GtkArg arg;
    GtkRange *range;
    arg.name = "adjustment";
    gtk_object_getv(GTK_OBJECT(slider),1,&arg);
    range = GTK_RANGE(GTK_VALUE_POINTER(arg));
    adj = gtk_range_get_adjustment(range);
    gtk_signal_connect(GTK_OBJECT(adj),"value_changed",
                       GTK_SIGNAL_FUNC(slider_moved),NULL);
  }
  effect_slider = glade_xml_get_widget(xml,"effect_slider");
  {
    GtkArg arg;
    GtkRange *range;
    arg.name = "adjustment";
    gtk_object_getv(GTK_OBJECT(effect_slider),1,&arg);
    range = GTK_RANGE(GTK_VALUE_POINTER(arg));
    effect_adj = gtk_range_get_adjustment(range);
    gtk_signal_connect(GTK_OBJECT(effect_adj),"value_changed",
                       GTK_SIGNAL_FUNC(effect_slider_moved),NULL);
    gtk_adjustment_set_value(effect_adj,
      gst_util_get_float_arg(GST_OBJECT(effect),effectname));
  }
g_print("have sliders\n");
  effect_name = glade_xml_get_widget(xml,"effect_name");
  gtk_entry_set_text(GTK_ENTRY(effect_name),effectname);
//  vumeter_progress = glade_xml_get_widget(xml,"vumeter");
  drawingarea = glade_xml_get_widget(xml,"drawingarea");
  g_print("drawingarea is %p\n",drawingarea);
g_print("have remainder\n");
  glade_xml_signal_autoconnect(xml);
g_print("autoconnected\n");

  /* set up initial GUI state */
  if (filename)
    gtk_entry_set_text(GTK_ENTRY(filename_entry),filename);

g_print("connecting signals\n");
  gtk_signal_connect(GTK_OBJECT(src),"eof",
                     GTK_SIGNAL_FUNC(eof),NULL);
  gtk_signal_connect(GTK_OBJECT(sink),"handoff",
                     GTK_SIGNAL_FUNC(handoff),NULL);

  g_print("DONE SETTING UP GUI\n");

  gtk_main();
  return 0;
}
