/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <gstesdsink.h>


/* EsdSink signals and args */
enum {
  HANDOFF,
  LAST_SIGNAL
};

enum {
  ARG_0,
  /* FILL ME */
};


static void gst_esdsink_class_init(GstEsdSinkClass *klass);
static void gst_esdsink_init(GstEsdSink *esdsink);


static GstFilterClass *parent_class = NULL;
static guint gst_esdsink_signals[LAST_SIGNAL] = { 0 };

static GstType *gst_esdsink_type_audio = NULL;

GtkType
gst_esdsink_get_type(void) {
  static GtkType esdsink_type = 0;

  if (!esdsink_type) {
    static const GtkTypeInfo esdsink_info = {
      "GstEsdSink",
      sizeof(GstEsdSink),
      sizeof(GstEsdSinkClass),
      (GtkClassInitFunc)gst_esdsink_class_init,
      (GtkObjectInitFunc)gst_esdsink_init,
      (GtkArgSetFunc)NULL,
      (GtkArgGetFunc)NULL,
      (GtkClassInitFunc)NULL,
    };
    esdsink_type = gtk_type_unique(GST_TYPE_FILTER,&esdsink_info);
  }

  if (gst_esdsink_type_audio == NULL) {
    gst_esdsink_type_audio = gst_type_get_by_name("audio");
//    g_print("got audio type\n");
  }

  return esdsink_type;
}

static void
gst_esdsink_class_init(GstEsdSinkClass *klass) {
  GtkObjectClass *gtkobject_class;

  gtkobject_class = (GtkObjectClass*)klass;

  parent_class = gtk_type_class(GST_TYPE_FILTER);

  gst_esdsink_signals[HANDOFF] =
    gtk_signal_new("handoff",GTK_RUN_LAST,gtkobject_class->type,
                   GTK_SIGNAL_OFFSET(GstEsdSinkClass,handoff),
                   gtk_marshal_NONE__POINTER_POINTER,GTK_TYPE_NONE,2,
                   GTK_TYPE_POINTER,GTK_TYPE_POINTER);
  gtk_object_class_add_signals(gtkobject_class,gst_esdsink_signals,
                               LAST_SIGNAL);
}

static void gst_esdsink_init(GstEsdSink *esdsink) {
  esdsink->sinkpad = gst_pad_new("sink",GST_PAD_SINK);
  gst_element_add_pad(GST_ELEMENT(esdsink),esdsink->sinkpad);
  if (gst_esdsink_type_audio == NULL) {
    gst_esdsink_type_audio = gst_type_get_by_name("audio");
    g_print("got audio type\n");
  }
  gst_pad_set_type_id(esdsink->sinkpad,
                      gst_type_get_id(gst_esdsink_type_audio));
  gst_pad_set_chain_function(esdsink->sinkpad,gst_esdsink_chain);

  esdsink->fd = open("/dev/dsp",O_WRONLY,0);

  if (esdsink->fd > 0) {
    /* set card state */
    esdsink->format = AFMT_S16_LE;
    esdsink->channels = 2;	/* stereo */
    esdsink->frequency = 44100;
    gst_esdsink_sync_parms(esdsink);
//    g_print("sound device set to format %d, %d channels, %dHz\n",
//            esdsink->format,esdsink->channels,esdsink->frequency);
  } else
    g_print("couldn't open audio device at this time\n");
}

void gst_esdsink_sync_parms(GstEsdSink *esdsink) {
  g_return_if_fail(esdsink != NULL);
  g_return_if_fail(GST_IS_ESDSINK(esdsink));
  g_return_if_fail(esdsink->fd > 0);

  ioctl(esdsink->fd,SNDCTL_DSP_SETFMT,&esdsink->format);
  ioctl(esdsink->fd,SNDCTL_DSP_CHANNELS,&esdsink->channels);
  ioctl(esdsink->fd,SNDCTL_DSP_SPEED,&esdsink->frequency);
}

GstElement *gst_esdsink_new(gchar *name) {
  GstElement *esdsink = GST_ELEMENT(gtk_type_new(GST_TYPE_ESDSINK));
  gst_object_set_name(GST_OBJECT(esdsink),name);
  return esdsink;
}

void gst_esdsink_chain(GstPad *pad,GstBuffer *buf) {
  GstEsdSink *esdsink;
  GstAudioMeta *meta;

  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(buf != NULL);
  g_return_if_fail(GST_IS_BUFFER(buf));

  /* this has to be an audio buffer */
//  g_return_if_fail(((GstMeta *)buf->meta)->type !=
//gst_esdsink_type_audio);

  esdsink = GST_ESDSINK(pad->parent);
//  g_print("gst_esdsink_chain: got buffer of %d bytes\n",
//          buf->datasize,gst_object_get_name(GST_OBJECT(esdsink)));

  meta = (GstAudioMeta *)buf->meta;
  if (meta != NULL) {
//    g_print("got audio packet with %d samples of %d %dKHz %s\n",
//            meta->samples,meta->format,meta->frequency,
//            (meta->channels == 2) ? "stereo" : "mono");
    if ((meta->format != esdsink->format) ||
        (meta->channels != esdsink->channels) ||
        (meta->frequency != esdsink->frequency)) {
      esdsink->format = meta->format;
      esdsink->channels = meta->channels;
      esdsink->frequency = meta->frequency;
      gst_esdsink_sync_parms(esdsink);
      g_print("sound device set to format %d, %d channels, %dHz\n",
              esdsink->format,esdsink->channels,esdsink->frequency);
    }
  } //else
//    g_print("don't have meta\n");

  gtk_signal_emit(GTK_OBJECT(esdsink),gst_esdsink_signals[HANDOFF],
                  esdsink);
  if (buf->data != NULL)
    write(esdsink->fd,buf->data,buf->datasize);

  gst_buffer_unref(buf);
}

void gst_esdsink_set_format(GstEsdSink *esdsink,gint format) {
  g_return_if_fail(esdsink != NULL);
  g_return_if_fail(GST_IS_ESDSINK(esdsink));

  esdsink->format = format;
  
  gst_esdsink_sync_parms(esdsink);
}

void gst_esdsink_set_channels(GstEsdSink *esdsink,gint channels) {
  g_return_if_fail(esdsink != NULL);
  g_return_if_fail(GST_IS_ESDSINK(esdsink));

  esdsink->channels = channels;

  gst_esdsink_sync_parms(esdsink);
}

void gst_esdsink_set_frequency(GstEsdSink *esdsink,gint frequency) {
  g_return_if_fail(esdsink != NULL);
  g_return_if_fail(GST_IS_ESDSINK(esdsink));

  esdsink->frequency = frequency;

  gst_esdsink_sync_parms(esdsink);
}
