/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gsttypefind.h>


/* TypeFind signals and args */
enum {
  HAVE_TYPE,
  LAST_SIGNAL
};

enum {
  ARG_0,
  TYPE,
};


static void gst_typefind_class_init(GstTypeFindClass *klass);
static void gst_typefind_init(GstTypeFind *typefind);


static GstSinkClass *parent_class = NULL;
static guint gst_typefind_signals[LAST_SIGNAL] = { 0 };

GtkType
gst_typefind_get_type(void) {
  static GtkType typefind_type = 0;

  if (!typefind_type) {
    static const GtkTypeInfo typefind_info = {
      "GstTypeFind",
      sizeof(GstTypeFind),
      sizeof(GstTypeFindClass),
      (GtkClassInitFunc)gst_typefind_class_init,
      (GtkObjectInitFunc)gst_typefind_init,
      (GtkArgSetFunc)gst_typefind_set_arg,
      (GtkArgGetFunc)gst_typefind_get_arg,
      (GtkClassInitFunc)NULL,
    };
    typefind_type = gtk_type_unique(GST_TYPE_SINK,&typefind_info);
  }
  return typefind_type;
}

static void
gst_typefind_class_init(GstTypeFindClass *klass) {
  GtkObjectClass *gtkobject_class;
  GstSinkClass *gstsink_class;

  gtkobject_class = (GtkObjectClass*)klass;
  gstsink_class = (GstSinkClass*)klass;

  parent_class = gtk_type_class(GST_TYPE_SINK);

  gtk_object_add_arg_type("GstTypeFind::type", GTK_TYPE_INT,
                          GTK_ARG_READABLE, ARG_TYPE);

  gtkobject_class->set_arg = gst_typefind_set_arg;
  gtkobject_class->get_arg = gst_typefind_get_arg;
}

static void gst_typefind_init(GstTypeFind *typefind) {
  typefind->sinkpad = gst_pad_new("sink",GST_PAD_SINK);
  gst_element_add_pad(GST_ELEMENT(typefind),typefind->sinkpad);
  gst_pad_set_chain_function(typefind->sinkpad,gst_typefind_chain);
}

GstElement *gst_typefind_new(gchar *name) {
  GstElement *typefind = GST_ELEMENT(gtk_type_new(GST_TYPE_TYPESINK));
  gst_object_set_name(GST_OBJECT(typefind),name);
  return typefind;
}

void gst_typefind_chain(GstPad *pad,GstBuffer *buf) {
  GstTypeFind *typefind;

  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(buf != NULL);
  g_return_if_fail(GST_IS_BUFFER(buf));

  typefind = GST_TYPESINK(pad->parent);
  g_print("gst_typefind_chain: got buffer of %d bytes in '%s'\n",
          buf->datasize,gst_object_get_name(GST_OBJECT(typefind)));
  gst_buffer_unref(buf);
}
