/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <gst/gstpad.h>
#include <gst/gsttype.h>


/* Pad signals and args */
enum {
  /* FILL ME */
  LAST_SIGNAL
};

enum {
  ARG_0,
  /* FILL ME */
};


static void gst_pad_class_init(GstPadClass *klass);
static void gst_pad_init(GstPad *pad);


static GstObject *parent_class = NULL;
static guint gst_pad_signals[LAST_SIGNAL] = { 0 };

GtkType
gst_pad_get_type(void) {
  static GtkType pad_type = 0;

  if (!pad_type) {
    static const GtkTypeInfo pad_info = {
      "GstPad",
      sizeof(GstPad),
      sizeof(GstPadClass),
      (GtkClassInitFunc)gst_pad_class_init,
      (GtkObjectInitFunc)gst_pad_init,
      (GtkArgSetFunc)NULL,
      (GtkArgGetFunc)NULL,
      (GtkClassInitFunc)NULL,
    };
    pad_type = gtk_type_unique(GST_TYPE_OBJECT,&pad_info);
  }
  return pad_type;
}

static void
gst_pad_class_init(GstPadClass *klass) {
  GtkObjectClass *gtkobject_class;

  gtkobject_class = (GtkObjectClass*)klass;

  parent_class = gtk_type_class(GST_TYPE_OBJECT);
}

static void gst_pad_init(GstPad *pad) {
}

/**
 * gst_pad_new:
 * @name: name of new pad
 * @direction: either GST_PAD_SRC or GST_PAD_SINK
 *
 * Create a new pad with given name.
 *
 * Returns: new pad
 */
GstPad *gst_pad_new(gchar *name,GstPadDirection direction) {
  GstPad *pad;

  g_return_if_fail(name != NULL);

  pad = GST_PAD(gtk_type_new(gst_pad_get_type()));
  pad->direction = direction;
  pad->name = g_strdup(name);
  return pad;
}

void gst_pad_destroy(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  
  gtk_object_destroy(GTK_OBJECT(pad));
}

GstPadDirection gst_pad_get_direction(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));

  return pad->direction;
}

void gst_pad_set_name(GstPad *pad,gchar *name) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(name != NULL);

  if (pad->name != NULL)
    g_free(pad->name);

  pad->name = g_strdup(name);
}

gchar *gst_pad_get_name(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));

  return pad->name;
}

void gst_pad_set_chain_function(GstPad *pad,GstPadChainFunction chain) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(chain != NULL);
  
  pad->chain = chain;
}

void gst_pad_push(GstPad *pad,GstBuffer *buffer) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(pad->peer != NULL);
  g_return_if_fail(pad->chain != NULL);
  g_return_if_fail(buffer != NULL);
  g_return_if_fail(GST_IS_BUFFER(buffer));

  gst_trace_add_entry(NULL,0,buffer,"push buffer");
  (pad->chain)(pad->peer,buffer);
}

void gst_pad_connect(GstPad *srcpad,GstPad *sinkpad) {
  GstPad *temppad;

  /* generic checks */
  g_return_if_fail(srcpad != NULL);
  g_return_if_fail(GST_IS_PAD(srcpad));
  g_return_if_fail(srcpad->peer == NULL);
  g_return_if_fail(sinkpad != NULL);
  g_return_if_fail(GST_IS_PAD(sinkpad));
  g_return_if_fail(sinkpad->peer == NULL);
  g_return_if_fail(sinkpad->chain != NULL);

  /* check for reversed directions and swap if necessary */
  if ((srcpad->direction == GST_PAD_SINK) &&
      (sinkpad->direction == GST_PAD_SRC)) {
    temppad = srcpad;
    srcpad = sinkpad;
    sinkpad = temppad;
  }
  g_return_if_fail((srcpad->direction == GST_PAD_SRC) &&
                   (sinkpad->direction == GST_PAD_SINK));

  /* first set peers */
  srcpad->peer = sinkpad;
  sinkpad->peer = srcpad;

  /* now copy the chain pointer from sink to src */
  srcpad->chain = sinkpad->chain;

  /* set the connected flag */
  /* FIXME: set connected flag */
}

void gst_pad_set_parent(GstPad *pad,GstObject *parent) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(pad->parent == NULL);
  g_return_if_fail(parent != NULL);
  g_return_if_fail(GTK_IS_OBJECT(parent));
  g_return_if_fail((gpointer)pad != (gpointer)parent);

  pad->parent = parent;
}

void gst_pad_set_ghost_parent(GstPad *pad,GstObject *parent) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(pad->ghostparent == NULL);
  g_return_if_fail(parent != NULL);
  g_return_if_fail(GTK_IS_OBJECT(parent));
  g_return_if_fail((gpointer)pad != (gpointer)parent);

  pad->ghostparent = parent;
}

GstObject *gst_pad_get_parent(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));

  return pad->parent;
}

GstObject *gst_pad_get_ghost_parent(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));

  return pad->ghostparent;
}

guint32 gst_pad_get_type_id(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));

  return pad->type;
}

void gst_pad_set_type_id(GstPad *pad,guint16 id) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(gst_type_find_by_id(id) != NULL);

  pad->type = id;
}

GstPad *gst_pad_get_peer(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));

  return pad->peer;
}

GstPadDirection gst_pad_get_directory(GstPad *pad) {
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));

  return pad->direction;
}

