/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <synaesthesia.h>


GstElementDetails gst_synaesthesia_details = {
  "Synaesthesia display",
  "Sink/Visualization",
  "Cool color display based on stereo info",
  VERSION,
  "Erik Walthinsen <omega@cse.ogi.edu>",
  "(C) 1999",
};


/* Synaesthesia signals and args */
enum {
  /* FILL ME */
  LAST_SIGNAL
};

enum {
  ARG_0,
  ARG_WIDTH,
  ARG_HEIGHT,
  ARG_WIDGET,
};


static void gst_synaesthesia_class_init(GstSynaesthesiaClass *klass);
static void gst_synaesthesia_init(GstSynaesthesia *synaesthesia);
static gboolean gst_synaesthesia_start(GstElement *element,
                                       GstElementState state);


static GstSinkClass *parent_class = NULL;
static guint gst_synaesthesia_signals[LAST_SIGNAL] = { 0 };
static void gst_synaesthesia_set_arg(GtkObject *object,GtkArg *arg,guint id);
static void gst_synaesthesia_get_arg(GtkObject *object,GtkArg *arg,guint id);


GtkType
gst_synaesthesia_get_type(void) {
  static GtkType synaesthesia_type = 0;

  if (!synaesthesia_type) {
    static const GtkTypeInfo synaesthesia_info = {
      "GstSynaesthesia",
      sizeof(GstSynaesthesia),
      sizeof(GstSynaesthesiaClass),
      (GtkClassInitFunc)gst_synaesthesia_class_init,
      (GtkObjectInitFunc)gst_synaesthesia_init,
      (GtkArgSetFunc)NULL,
      (GtkArgGetFunc)NULL,
      (GtkClassInitFunc)NULL,
    };
    synaesthesia_type = gtk_type_unique(GST_TYPE_SINK,&synaesthesia_info);
  }
  return synaesthesia_type;
}

static void gst_synaesthesia_class_init(GstSynaesthesiaClass *klass) {
  GtkObjectClass *gtkobject_class;
  GstElementClass *gstelement_class;
  GstSinkClass *gstsink_class;

  gtkobject_class = (GtkObjectClass*)klass;
  gstelement_class = (GstElementClass*)klass;
  gstsink_class = (GstSinkClass*)klass;

  parent_class = gtk_type_class(GST_TYPE_SINK);

  gtk_object_add_arg_type("GstSynaesthesia::width", GTK_TYPE_INT,
                          GTK_ARG_READABLE, ARG_WIDTH);
  gtk_object_add_arg_type("GstSynaesthesia::height", GTK_TYPE_INT,
                          GTK_ARG_READABLE, ARG_HEIGHT);
  gtk_object_add_arg_type("GstSynaesthesia::widget", GTK_TYPE_WIDGET,
                          GTK_ARG_READABLE, ARG_WIDGET);

  gstelement_class->start = gst_synaesthesia_start;

  gtkobject_class->set_arg = gst_synaesthesia_set_arg;
  gtkobject_class->get_arg = gst_synaesthesia_get_arg;
}

static void gst_synaesthesia_init(GstSynaesthesia *synaesthesia) {
  synaesthesia->sinkpad = gst_pad_new("sink",GST_PAD_SINK);
  gst_element_add_pad(GST_ELEMENT(synaesthesia),synaesthesia->sinkpad);
  gst_pad_set_chain_function(synaesthesia->sinkpad,gst_synaesthesia_chain);
  synaesthesia->srcpad = gst_pad_new("src",GST_PAD_SRC);
  gst_element_add_pad(GST_ELEMENT(synaesthesia),synaesthesia->srcpad);

  synaesthesia->meta = NULL;
  synaesthesia->width = 150;
  synaesthesia->height = 50;
}

GstElement *gst_synaesthesia_new(gchar *name) {
  GstElement *synaesthesia = GST_ELEMENT(gtk_type_new(GST_TYPE_SYNAESTHESIA));
  gst_object_set_name(GST_OBJECT(synaesthesia),name);
  return synaesthesia;
}

void gst_synaesthesia_chain(GstPad *pad,GstBuffer *buf) {
  GstSynaesthesia *syna;
  gint16 *samples;
  gint samplecount,i;
  gint max;

  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(buf != NULL);
  g_return_if_fail(GST_IS_BUFFER(buf));

  syna = GST_SYNAESTHESIA(pad->parent);

  /* first deal with audio metadata */
  if (buf->meta) {
    if (syna->meta != NULL) {
      /* FIXME: need to unref the old metadata so it goes away */
    }
    /* we just make a copy of the pointer */
    syna->meta = (MetaAudioRaw *)GST_BUFFER_META(buf);
    /* FIXME: now we have to ref the metadata so it doesn't go away */
  }

  g_return_if_fail(syna->meta != NULL);

  samples = (gint16 *)GST_BUFFER_DATA(buf);
  samplecount = GST_BUFFER_DATASIZE(buf) /
                (syna->meta->channels * sizeof(gint16));

//  DEBUG("fading\n");
//  fade(&syna->sp);
  DEBUG("doing effect\n");
  coreGo(&syna->sp,GST_BUFFER_DATA(buf),samples);

//  DEBUG("drawing\n");
/*  DEBUG("gdk_draw_indexed_image(%p,%p,%d,%d,%d,%d,%s,%p,%d,%p);\n",
        syna->image->window,
	syna->image->style->fg_gc[GTK_STATE_NORMAL],
	0,0,syna->width,syna->height,
	"GDK_RGB_DITHER_NORMAL",
	syna->sp.output,syna->width,
	&syna->cmap);*/
/*  gdk_draw_indexed_image(syna->image->window,
	syna->image->style->fg_gc[GTK_STATE_NORMAL],
	0,0,syna->width,syna->height,
	GDK_RGB_DITHER_NORMAL,
	syna->sp.output,syna->width,
	&syna->cmap);*/
  gdk_draw_gray_image(syna->image->window,
	syna->image->style->fg_gc[GTK_STATE_NORMAL],
	0,0,syna->width,syna->height,
	GDK_RGB_DITHER_NORMAL,
	syna->sp.output,syna->width);

  gst_trace_add_entry(NULL,0,buf,"synaesthesia: calculated syna");

  gst_buffer_unref(buf);
}

static void gst_synaesthesia_set_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstSynaesthesia *synaesthesia;

  /* it's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_SYNAESTHESIA(object));
  synaesthesia = GST_SYNAESTHESIA(object);

  switch(id) {
    ARG_WIDTH:
      synaesthesia->width = GTK_VALUE_INT(*arg);
      break;
    ARG_HEIGHT:
      synaesthesia->height = GTK_VALUE_INT(*arg);
      break;
    default:
      break;
  }
}

static void gst_synaesthesia_get_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstSynaesthesia *synaesthesia;

  /* it's not null if we got it, but it might not be ours */
  synaesthesia = GST_SYNAESTHESIA(object);

  DEBUG("have synaesthesia get_arg(%d), wanting %d\n",id,ARG_WIDGET);

  switch(id) {
    case ARG_WIDTH: {
      GTK_VALUE_INT(*arg) = synaesthesia->width;
      DEBUG("returning width value %s\n",GTK_VALUE_INT(*arg));
      break;
    }
    case ARG_HEIGHT: {
      GTK_VALUE_INT(*arg) = synaesthesia->height;
      DEBUG("returning height value %s\n",GTK_VALUE_INT(*arg));
      break;
    }
    case ARG_WIDGET: {
      GTK_VALUE_OBJECT(*arg) = GTK_OBJECT(synaesthesia->image);
      DEBUG("returning widget value %p\n",GTK_VALUE_OBJECT(*arg));
      break;
    }
    default: {
      arg->type = GTK_TYPE_INVALID;
      DEBUG("returning invalid type\n");
      break;
    }
  }
}



GstPlugin *plugin_init(GModule *module) {
  GstPlugin *plugin;
  GstElementFactory *factory;
  guint16 rawtype;

  if (gst_plugin_find("synaesthesia") != NULL) return NULL;

  plugin = gst_plugin_new("synaesthesia");
  g_return_if_fail(plugin != NULL);

  /* create an elementfactory for the synaesthesia element */
  factory = gst_elementfactory_new("synaesthesia",GST_TYPE_SYNAESTHESIA,
                                   &gst_synaesthesia_details);
  g_return_if_fail(factory != NULL);
  gst_plugin_add_factory(plugin,factory);

  /* this sink needs raw audio */
  rawtype = gst_type_find_by_mime("audio/raw");
  gst_type_add_sink(rawtype,factory);

  g_print("synaesthesia: added 'synaesthesia' element\n");

  return plugin;
}

static gboolean gst_synaesthesia_start(GstElement *element,
                                       GstElementState state) {
  GstSynaesthesia *syna;
  gint i;

  g_return_if_fail(GST_IS_SYNAESTHESIA(element));

  syna = GST_SYNAESTHESIA(element);

  syna->width = 100;
  syna->height = 50;
  syna->starsize = 0.125;

  coreInit(&syna->sp,syna->width,syna->height);
  setStarSize(&syna->sp,syna->starsize);

  setupPalette(&syna->sp,syna->cmap.colors);

  gdk_rgb_init();
  syna->image = gtk_drawing_area_new();
  DEBUG("image is %p\n",syna->image);
  gtk_drawing_area_size(GTK_DRAWING_AREA(syna->image),syna->width,
                        syna->height);
  gtk_widget_show(syna->image);

  DEBUG("started synaesthesia\n");
  return TRUE;
}
