/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <vumeter.h>


GstElementDetails gst_vumeter_details = {
  "VU Meter",
  "Sink/Meter",
  "Simple volume indicator",
  VERSION,
  "Erik Walthinsen <omega@cse.ogi.edu>",
  "(C) 1999",
};


/* VuMeter signals and args */
enum {
  /* FILL ME */
  LAST_SIGNAL
};

enum {
  ARG_0,
  ARG_VOLUME,
  ARG_VOLUMEPTR,
};


static void gst_vumeter_class_init(GstVuMeterClass *klass);
static void gst_vumeter_init(GstVuMeter *vumeter);


static GstSinkClass *parent_class = NULL;
static guint gst_vumeter_signals[LAST_SIGNAL] = { 0 };
static void gst_vumeter_set_arg(GtkObject *object,GtkArg *arg,guint id);
static void gst_vumeter_get_arg(GtkObject *object,GtkArg *arg,guint id);


GtkType
gst_vumeter_get_type(void) {
  static GtkType vumeter_type = 0;

  if (!vumeter_type) {
    static const GtkTypeInfo vumeter_info = {
      "GstVuMeter",
      sizeof(GstVuMeter),
      sizeof(GstVuMeterClass),
      (GtkClassInitFunc)gst_vumeter_class_init,
      (GtkObjectInitFunc)gst_vumeter_init,
      (GtkArgSetFunc)gst_vumeter_set_arg,
      (GtkArgGetFunc)gst_vumeter_get_arg,
      (GtkClassInitFunc)NULL,
    };
    vumeter_type = gtk_type_unique(GST_TYPE_SINK,&vumeter_info);
  }
  return vumeter_type;
}

static void gst_vumeter_class_init(GstVuMeterClass *klass) {
  GtkObjectClass *gtkobject_class;
  GstSinkClass *gstsink_class;

  gtkobject_class = (GtkObjectClass*)klass;
  gstsink_class = (GstSinkClass*)klass;

  parent_class = gtk_type_class(GST_TYPE_SINK);

  gtk_object_add_arg_type("GstVuMeter::volume", GTK_TYPE_INT,
                          GTK_ARG_READABLE, ARG_VOLUME);
  gtk_object_add_arg_type("GstVuMeter::volumeptr", GTK_TYPE_POINTER,
                          GTK_ARG_READABLE, ARG_VOLUMEPTR);

  gtkobject_class->set_arg = gst_vumeter_set_arg;
  gtkobject_class->get_arg = gst_vumeter_get_arg;
}

static void gst_vumeter_init(GstVuMeter *vumeter) {
  vumeter->sinkpad = gst_pad_new("sink",GST_PAD_SINK);
  gst_element_add_pad(GST_ELEMENT(vumeter),vumeter->sinkpad);
  gst_pad_set_chain_function(vumeter->sinkpad,gst_vumeter_chain);
  vumeter->srcpad = gst_pad_new("src",GST_PAD_SRC);
  gst_element_add_pad(GST_ELEMENT(vumeter),vumeter->srcpad);

  vumeter->meta = NULL;
  vumeter->volume = 0.0;
}

GstElement *gst_vumeter_new(gchar *name) {
  GstElement *vumeter = GST_ELEMENT(gtk_type_new(GST_TYPE_VUMETER));
  gst_object_set_name(GST_OBJECT(vumeter),name);
  return vumeter;
}

void gst_vumeter_chain(GstPad *pad,GstBuffer *buf) {
  GstVuMeter *vumeter;
  gint16 *samples;
  gint samplecount,i;
  gint max;

  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(buf != NULL);
  g_return_if_fail(GST_IS_BUFFER(buf));

  vumeter = GST_VUMETER(pad->parent);

  /* first deal with audio metadata */
  if (buf->meta) {
    if (vumeter->meta != NULL) {
      /* FIXME: need to unref the old metadata so it goes away */
    }
    /* we just make a copy of the pointer */
    vumeter->meta = (MetaAudioRaw *)GST_BUFFER_META(buf);
    /* FIXME: now we have to ref the metadata so it does go away */
  }

  g_return_if_fail(vumeter->meta != NULL);

/* FIXME: this is very s16_le specific!!! */
  samples = (gint16 *)GST_BUFFER_DATA(buf);
  samplecount = GST_BUFFER_DATASIZE(buf) / 4;
//                (vumeter->meta->channels * sizeof(gint16));

  max = 0;
  for (i=0;i<samplecount;i++) {
    if (samples[i] > max) max = samples[i];
    if ((-samples[i]) > max) max = (-samples[i]);
  }

  gst_trace_add_entry(NULL,0,buf,"vumeter: calculated volume");

  vumeter->volume = max;
//  g_print("current volume is %d\n",vumeter->volume);

  gst_buffer_unref(buf);
}

static void gst_vumeter_set_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstVuMeter *vumeter;

  /* it's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_VUMETER(object));
  vumeter = GST_VUMETER(object);

  switch(id) {
    default:
      break;
  }
}

static void gst_vumeter_get_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstVuMeter *vumeter;

//  g_print("in gst_vumeter_get_arg, id %d, VOLUME is
//%d\n",id,ARG_VOLUME);
  /* it's not null if we got it, but it might not be ours */
//  vumeter = GST_VUMETER(object);
  vumeter = (GstVuMeter *)object;

//  switch(id) {
//    ARG_VOLUME:
      GTK_VALUE_INT(*arg) = vumeter->volume;
//      g_print("returning value of %d\n",GTK_VALUE_INT(*arg));
/*      break;
    ARG_VOLUMEPTR:
      GTK_VALUE_POINTER(*arg) = &vumeter->volume;
      break;
    default:
      arg->type = GTK_TYPE_INVALID;
      break;
  }*/
}



GstPlugin *plugin_init(GModule *module) {
  GstPlugin *plugin;
  GstElementFactory *factory;
  guint16 rawtype;

  if (gst_plugin_find("vumeter") != NULL) return NULL;

  plugin = gst_plugin_new("vumeter");
  g_return_if_fail(plugin != NULL);

  /* create an elementfactory for the vumeter element */
  factory = gst_elementfactory_new("vumeter",GST_TYPE_VUMETER,
                                   &gst_vumeter_details);
  g_return_if_fail(factory != NULL);
  gst_plugin_add_factory(plugin,factory);
  g_print("vumeter: added 'vumeter' element\n");

  /* this sink needs raw audio */
  rawtype = gst_type_find_by_mime("audio/raw");
  gst_type_add_sink(rawtype,factory);

  return plugin;
}

